@@ -15,13 +15,15 @@
 ###
 
 # -*- coding: utf-8 -*-
-""" Set Command for RDMC """
+"""Set Command for RDMC"""
 import json
 import os
 import sys
 
 import redfish.ris
 
+from redfish.rest.connections import SecurityStateError
+
 try:
     from rdmc_helper import (
         InvalidCommandLineError,
@@ -60,6 +62,7 @@ class SetCommand:
                 "RebootCommand",
                 "RawPatchCommand",
                 "SelectCommand",
+                "LogoutCommand",
             ],
         }
         self.cmdbase = None
@@ -87,11 +90,34 @@ class SetCommand:
             raise InvalidCommandLineError("The 'set' command is not useful in " "non-interactive and non-cache modes.")
 
         self.setvalidation(options)
-        _ = args[0].find("HighSecurity")
-
+        ilover = self.rdmc.app.getiloversion()
+        if ilover >= 7.000 and ("HighSecurity" in args[0] or "Production" in args[0]):
+            self.rdmc.ui.error(
+                "HighSecurity and Production modes are not supported on iLO 7 , Please use SecureStandard instead \n"
+            )
+            raise SecurityStateError(
+                "HighSecurity and Production modes are not supported on iLO 7 , Please use SecureStandard instead"
+            )
+        else:
+            if "SecureStandard" in args[0] and ilover < 7.000:
+                self.rdmc.ui.error(
+                    "SecureStandard mode is not supported on iLO 6 and below , Please use HighSecurity instead \n"
+                )
+                raise SecurityStateError(
+                    "SecureStandard mode is not supported on iLO 6 and below , Please use HighSecurity instead \n"
+                )
+            _ = args[0].find("HighSecurity")
+        if ilover >= 7.000:
+            _ = args[0].find("SecureStandard")
         fsel = None
         fval = None
         if args:
+            if "ethernetinterface" in self.rdmc.app.selector.lower() and "ipv4addresses" in args[0].lower():
+                raise InvalidCommandLineError(
+                    "For security reasons this request is discarded. "
+                    "Use `rawpatch` or the `load --force_network_config` command "
+                    "to modify the IP configuration of the ilO.\n"
+                )
             if options.filter:
                 try:
                     (fsel, fval) = str(options.filter).strip("'\" ").split("=")
@@ -205,8 +231,8 @@ class SetCommand:
             if options.reboot and not options.commit:
                 self.auxcommands["reboot"].run(options.reboot)
 
-            # if options.logout:
-            #    self.logoutobj.run("")
+            if options.logout:
+                self.auxcommands["logout"].run("")
 
         else:
             raise InvalidCommandLineError("Missing parameters for 'set' command.\n")
@@ -218,6 +244,9 @@ class SetCommand:
         :param skipprint: boolean to determine output
         :type skipprint: boolean.
         """
+        import platform
+        import tempfile
+
         try:
             (options, args) = self.rdmc.rdmc_parse_arglist(self, line)
         except (InvalidCommandLineErrorOPTS, SystemExit):
@@ -277,7 +306,23 @@ class SetCommand:
                         val = int(val)
                 patch_data = dict()
                 payload = {newargs[-1]: val} if newargs else {sel: val}
-                if newargs:
+                if "drive" in self.rdmc.app.selector.lower():
+                    tempdir = "/tmp" if platform.system() == "Darwin" else tempfile.gettempdir()
+                    temp_file = os.path.join(tempdir, "temp_patch.json")
+                    out_file = open(temp_file, "w")
+                    if getattr(options, "filter", None):
+                        filt_arr = options.filter.split("=")
+                        patchpath = filt_arr[-1]
+                    else:
+                        sys.stdout.write("Please provide the actual drive to modify using filter option\n")
+                        return
+                    patch_payload = {patchpath: payload}
+                    json.dump(patch_payload, out_file, indent=6)
+                    out_file.close()
+                    sys.stdout.write("payload %s \n" % patch_payload)
+                    self.auxcommands["rawpatch"].run(temp_file + " --service")
+                    os.remove(temp_file)
+                elif newargs:
                     for key in newargs[:-1][::-1]:
                         if key == "PowerControl":
                             payload = {key: [payload]}
@@ -285,10 +330,10 @@ class SetCommand:
                             payload = {key: payload}
                         patch_data.update(payload)
                         if "Oem" in key:
-                            if "managernetworkprotocol." or "thermal." in self.rdmc.app.selector:
-                                import platform
-                                import tempfile
-
+                            if (
+                                "managernetworkprotocol." in self.rdmc.app.selector
+                                or "thermal." in self.rdmc.app.selector
+                            ):
                                 tempdir = "/tmp" if platform.system() == "Darwin" else tempfile.gettempdir()
                                 temp_file = os.path.join(tempdir, "temp_patch.json")
                                 out_file = open(temp_file, "w")
@@ -332,11 +377,15 @@ class SetCommand:
             ("Oem/Hpe/EnhancedDownloadPerformanceEnabled" in line[0])
             or ("Oem/Hpe/ThermalConfiguration" in line[0])
             or ("Oem/Hpe/FanPercentMinimum" in line[0])
-            or ("Power" in line[0])
+            or ("PowerControl" in line[0])
+            or (self.rdmc.app.selector is not None and "drive" in self.rdmc.app.selector.lower())
         ):
             self.patchfunction(line)
         else:
-            self.setfunction(line, skipprint=skipprint)
+            try:
+                self.setfunction(line, skipprint=skipprint)
+            except SecurityStateError:
+                return ReturnCodes.V1_SECURITY_STATE_ERROR
         # Return code
         return ReturnCodes.SUCCESS
 
