@@ -15,7 +15,7 @@
 # ##
 
 # -*- coding: utf-8 -*-
-""" Fwpkg Command for rdmc """
+"""Fwpkg Command for rdmc"""
 
 import os
 import json
@@ -40,6 +40,7 @@ try:
         UploadError,
         TaskQueueError,
         FirmwareUpdateError,
+        FlashUnsupportedByIloError,
     )
 except ImportError:
     from ilorest.rdmc_helper import (
@@ -51,6 +52,7 @@ except ImportError:
         UploadError,
         TaskQueueError,
         FirmwareUpdateError,
+        FlashUnsupportedByIloError,
     )
 
 
@@ -95,6 +97,8 @@ class FwpkgCommand:
             self.parser.print_help()
             return ReturnCodes.SUCCESS
         try:
+            if "-" in line[0] and "thirdparty" in line[0]:
+                line.insert(0, "dummy_arg")
             (options, _) = self.rdmc.rdmc_parse_arglist(self, line)
         except (InvalidCommandLineErrorOPTS, SystemExit):
             if ("-h" in line) or ("--help" in line):
@@ -112,11 +116,61 @@ class FwpkgCommand:
             raise IncompatibleiLOVersionError(
                 "Please upgrade to iLO 5 1.20 or greater to ensure correct flash of this firmware."
             )
-        tempdir = ""
+
+        updateservice_url = "/redfish/v1/UpdateService/"
+        # Get handler to check Accept3rdPartyFirmware enabled or disabled
+        get_content = self.rdmc.app.get_handler(updateservice_url, silent=True, service=True).dict
+
+        accept3rdpartyfw = ""
         if (
-            not options.fwpkg.endswith(".fwpkg")
-            and not options.fwpkg.lower().endswith(".fup")
-            and not options.fwpkg.lower().endswith(".hpb")
+            get_content
+            and "Oem" in get_content
+            and "Hpe" in get_content["Oem"]
+            and "Accept3rdPartyFirmware" in get_content["Oem"]["Hpe"]
+        ):
+            accept3rdpartyfw = get_content["Oem"]["Hpe"]["Accept3rdPartyFirmware"]
+
+        line_l = line[0].lower()
+        if (
+            line
+            and ".fwpkg" not in line_l
+            and (".pup" in line_l or ".hpb" in line_l or ".fup" in line_l or ".bin" in line_l)
+        ):
+            if not accept3rdpartyfw and not options.enable_thirdparty_fw:
+                raise InvalidCommandLineErrorOPTS(
+                    "Please use the option --enable_thirdparty_fw to flash Third "
+                    "Party Firmware Packages.\nBy default only .fwpkg files are allowed.\n"
+                )
+
+        if options.enable_thirdparty_fw:
+            self.rdmc.ui.printer("Enabling Third party Firmware flashing.\n")
+            self.rdmc.app.patch_handler(
+                updateservice_url,
+                {"Oem": {"Hpe": {"Accept3rdPartyFirmware": True}}},
+                silent=True,
+                service=True,
+            )
+            if "dummy" in line[0]:
+                return ReturnCodes.SUCCESS
+        elif options.disable_thirdparty_fw:
+            self.rdmc.ui.printer("Disabling Third party Firmware flashing.\n")
+            self.rdmc.app.patch_handler(
+                updateservice_url,
+                {"Oem": {"Hpe": {"Accept3rdPartyFirmware": False}}},
+                silent=True,
+                service=True,
+            )
+            if "dummy" in line[0]:
+                return ReturnCodes.SUCCESS
+
+        tempdir = ""
+        fwpkg_l = options.fwpkg.lower()
+        if (
+            not fwpkg_l.endswith(".fwpkg")
+            and not fwpkg_l.endswith(".fup")
+            and not fwpkg_l.endswith(".hpb")
+            and not fwpkg_l.endswith(".bin")
+            and not fwpkg_l.endswith(".pup")
         ):
             LOGGER.error("Invalid file type. Please make sure the file provided is a valid .fwpkg file type.")
             raise InvalidFileInputError(
@@ -124,18 +178,22 @@ class FwpkgCommand:
             )
 
         try:
-            components, tempdir, comptype, _ = self.preparefwpkg(self, options.fwpkg)
+            components, tempdir, comptype, _ = self.preparefwpkg(self, options.fwpkg, command_name="flashfwpkg")
             if comptype == "D":
                 LOGGER.error("Component Type D, Unable to flash this fwpkg file.")
                 raise InvalidFileInputError("Unable to flash this fwpkg file.")
 
             elif options.targets:
                 for component in components:
+                    component_l = component.lower()
                     if (
-                        component.endswith(".fwpkg")
-                        or component.lower().endswith(".hpb")
-                        or component.lower().endswith(".fup")
-                        or component.endswith(".zip")
+                        component_l.endswith(".fwpkg")
+                        or component_l.endswith(".hpb")
+                        or component_l.endswith(".HPb")
+                        or component_l.endswith(".bin")
+                        or component_l.endswith(".fup")
+                        or component_l.endswith(".pup")
+                        or component_l.endswith(".zip")
                     ):
                         uploadcommand = "--component %s" % component
                     else:
@@ -202,11 +260,11 @@ class FwpkgCommand:
                 )
             elif comptype in ["C", "BC"]:
                 message = "This firmware is set to flash on reboot.\n"
-
-            if "blobstore" not in self.rdmc.app.redfishinst.base_url:
-                if not self.auxcommands["uploadcomp"].wait_for_state_change():
-                    # Failed to upload the component.
-                    raise FirmwareUpdateError("Error while processing the component.")
+            if ".bin" not in components[0]:
+                if "blobstore" not in self.rdmc.app.redfishinst.base_url:
+                    if not self.auxcommands["uploadcomp"].wait_for_state_change():
+                        # Failed to upload the component.
+                        raise FirmwareUpdateError("Error while processing the component.")
 
             self.rdmc.ui.printer(message)
 
@@ -258,14 +316,15 @@ class FwpkgCommand:
                 "are finished processing. Use the taskqueue command to monitor updates.\n"
             )
 
-    def get_comp_type(self, payload):
-        """Get's the component type and returns it
+    def get_comp_type(self, payload, command_name="flashfwpkg"):
+        """Gets the component type and returns it
 
         :param payload: json payload of .fwpkg file
         :type payload: dict.
         :returns: returns the type of component. Either A,B,C, or D.
         :rtype: string
         """
+        ilo_ver_int = self.rdmc.app.getiloversion()
         ctype = ""
         if "Uefi" in payload["UpdatableBy"] and "RuntimeAgent" in payload["UpdatableBy"]:
             ctype = "D"
@@ -278,14 +337,36 @@ class FwpkgCommand:
                 type_set = None
                 for fw in data:
                     for device in payload["Devices"]["Device"]:
-                        if fw["Oem"]["Hpe"].get("Targets") is not None:
-                            if device["Target"] in fw["Oem"]["Hpe"]["Targets"]:
-                                if "Slot=" in fw["Oem"]["Hpe"]["DeviceContext"]:
-                                    cc_flag = True
-                                elif "N/A" in fw["Oem"]["Hpe"]["DeviceContext"]:
-                                    da_flag = True
-                                elif "Slot=" not in fw["Oem"]["Hpe"]["DeviceContext"]:
-                                    da_flag = True
+                        if (
+                            fw["Oem"]["Hpe"].get("Targets") is not None
+                            and device["Target"] in fw["Oem"]["Hpe"]["Targets"]
+                        ):
+                            if fw["Oem"]["Hpe"].get("DeviceContext") is None:
+                                LOGGER.error("DeviceContext is not found, please wait for a while & try again.")
+                                raise UploadError("DeviceContext is not found, please wait for a while & try again.")
+                            else:
+                                # print device context in debugger
+                                LOGGER.info("DeviceContext {}".format(fw["Oem"]["Hpe"]["DeviceContext"]))
+                                if (
+                                    "Slot=" in fw["Oem"]["Hpe"]["DeviceContext"]
+                                    and ":" in fw["Oem"]["Hpe"]["DeviceContext"]
+                                ):
+                                    if (
+                                        (ilo_ver_int >= 6.169 and ilo_ver_int < 7.000) or ilo_ver_int >= 7.113
+                                    ) and command_name != "uploadcomp":
+                                        if fw["Updateable"]:
+                                            cc_flag = True
+                                        else:
+                                            raise FlashUnsupportedByIloError(
+                                                "The flashing of this component " "is not supported by iLO.\n"
+                                            )
+                                    else:
+                                        cc_flag = True
+                                else:
+                                    if fw["Updateable"]:
+                                        cc_flag = True
+                                    else:
+                                        da_flag = True
                 if cc_flag and da_flag:
                     ctype = "BC"
                     type_set = True
@@ -297,7 +378,7 @@ class FwpkgCommand:
                     type_set = True
                 if type_set is None:
                     LOGGER.error("Component type is not identified, Please check if the particular H/W is present")
-                    ilo_ver_int = self.rdmc.app.getiloversion()
+                    # ilo_ver_int = self.rdmc.app.getiloversion()
                     ilo_ver = str(ilo_ver_int)
                     error_msg = (
                         "Cannot flash the component on this server, check whether the component is fwpkg-v2 "
@@ -315,14 +396,16 @@ class FwpkgCommand:
         else:
             for device in payload["Devices"]["Device"]:
                 for image in device["FirmwareImages"]:
-                    if "DirectFlashOk" not in list(image.keys()):
+                    flash_key = next((key for key in image.keys() if key.lower() == "directflashok"), None)
+
+                    if flash_key is None:
                         raise InvalidFileInputError("Cannot flash this firmware.")
-                    if image["DirectFlashOk"]:
+                    if flash_key and image[flash_key]:
                         ctype = "A"
                         if image["ResetRequired"]:
                             ctype = "B"
                             break
-                    elif image["UefiFlashable"]:
+                    elif image.get("UefiFlashable", image.get("UEFIFlashable", False)):
                         ctype = "C"
                         break
                     else:
@@ -331,7 +414,49 @@ class FwpkgCommand:
         return ctype
 
     @staticmethod
-    def preparefwpkg(self, pkgfile):
+    def check_decoupled(self, pkgfile):
+        """check for decoupled json
+
+        :param pkgfile: Location of the .fwpkg file
+        :type pkgfile: string.
+        :returns: returns the flag for flashing and decoupling
+        :rtype: bool, bool, string
+        """
+        if os.path.dirname(pkgfile):  # This means it's not just a file name
+            base_dir = os.path.dirname(pkgfile)
+            package_file = os.path.basename(pkgfile)
+        else:
+            base_dir = os.getcwd()
+            package_file = pkgfile
+
+        package_name, _ = os.path.splitext(package_file)
+
+        # Construct the expected path for the JSON file
+        json_file_name = f"{package_name}.json"
+        json_file_path = os.path.join(base_dir, json_file_name)
+
+        # List all files in the base directory and check for a case-insensitive match
+        decoupled_flag = False
+        payload_file = None
+
+        # List all files in the base directory
+        files_in_dir = os.listdir(base_dir)
+
+        # Check if any file in the directory matches the JSON file name case-insensitively
+        for file in files_in_dir:
+            if file.lower() == json_file_name.lower():
+                # File found, set flag and read the file
+                decoupled_flag = True
+                json_file_path = os.path.join(base_dir, file)  # Update json_file_path to match the case of the file
+                with open(json_file_path, "r", encoding="utf-8") as json_file:
+                    payload_file = json.load(json_file)
+                break  # Exit once the file is found
+
+        # If the file was not found , decoupled_flag remains False
+        return decoupled_flag, payload_file
+
+    @staticmethod
+    def preparefwpkg(self, pkgfile, command_name="flashfwpkg"):
         """Prepare fwpkg file for flashing
 
         :param pkgfile: Location of the .fwpkg file
@@ -345,25 +470,42 @@ class FwpkgCommand:
         payloaddata = None
         tempdir = tempfile.mkdtemp()
         pldmflag = False
-        if not pkgfile.lower().endswith(".fup") and not pkgfile.lower().endswith(".hpb"):
-            try:
-                zfile = zipfile.ZipFile(pkgfile)
-                zfile.extractall(tempdir)
-                zfile.close()
-            except Exception as excp:
-                raise InvalidFileInputError("Unable to unpack file. " + str(excp))
+        pkgfile_l = pkgfile.lower()
+        if (
+            not pkgfile_l.endswith(".fup")
+            and not pkgfile_l.endswith(".hpb")
+            and not pkgfile_l.endswith(".bin")
+            and not pkgfile_l.endswith(".pup")
+        ):
+            decoupled_flag, payload_file = self.auxcommands["flashfwpkg"].check_decoupled(self, pkgfile)
+            if not decoupled_flag:
+                try:
+                    zfile = zipfile.ZipFile(pkgfile)
+                    zfile.extractall(tempdir)
+                    zfile.close()
+                except Exception as excp:
+                    raise InvalidFileInputError("Unable to unpack file. " + str(excp))
 
-            files = os.listdir(tempdir)
+                files = os.listdir(tempdir)
 
-            if "payload.json" in files:
-                with open(os.path.join(tempdir, "payload.json"), encoding="utf-8") as pfile:
-                    data = pfile.read()
-                payloaddata = json.loads(data)
+                if "payload.json" in files:
+                    with open(os.path.join(tempdir, "payload.json"), encoding="utf-8") as pfile:
+                        data = pfile.read()
+                    payloaddata = json.loads(data)
+                else:
+                    raise InvalidFileInputError(
+                        "Invalid FWPKG component due to missing component metadata. \nPlease download the json file "
+                        "along with FWPKG component or provide FWPKG component having payload.json file"
+                    )
             else:
-                raise InvalidFileInputError("Unable to find payload.json in fwpkg file.")
-
-        if not pkgfile.lower().endswith(".fup") and not pkgfile.lower().endswith(".hpb"):
-            comptype = self.auxcommands["flashfwpkg"].get_comp_type(payloaddata)
+                payloaddata = payload_file
+        if (
+            not pkgfile_l.endswith(".fup")
+            and not pkgfile_l.endswith(".hpb")
+            and not pkgfile_l.endswith(".bin")
+            and not pkgfile_l.endswith(".pup")
+        ):
+            comptype = self.auxcommands["flashfwpkg"].get_comp_type(payloaddata, command_name)
         else:
             comptype = "A"
 
@@ -371,7 +513,12 @@ class FwpkgCommand:
         if comptype in ["C", "BC"]:
             imagefiles = [self.auxcommands["flashfwpkg"].type_c_change(tempdir, pkgfile)]
         else:
-            if not pkgfile.lower().endswith("fup") and not pkgfile.lower().endswith(".hpb"):
+            if (
+                not pkgfile_l.endswith("fup")
+                and not pkgfile_l.endswith(".hpb")
+                and not pkgfile_l.endswith(".bin")
+                and not pkgfile_l.endswith(".pup")
+            ):
                 for device in payloaddata["Devices"]["Device"]:
                     for firmwareimage in device["FirmwareImages"]:
                         if "PLDMImage" in firmwareimage and firmwareimage["PLDMImage"]:
@@ -396,8 +543,14 @@ class FwpkgCommand:
             if dll.isFwpkg20(fwpkg_buffer, 2048):
                 imagefiles = [pkgfile]
                 tempdir = ""
-        if pkgfile.lower().endswith(".fup") or pkgfile.lower().endswith(".hpb"):
+        if (
+            pkgfile_l.endswith(".hpb")
+            or pkgfile_l.endswith(".fup")
+            or pkgfile_l.endswith(".pup")
+            or pkgfile_l.endswith(".bin")
+        ):
             imagefiles = [pkgfile]
+
         elif self.rdmc.app.getiloversion() > 5.230 and payloaddata.get("PackageFormat") == "FWPKG-v2":
             imagefiles = [pkgfile]
         return imagefiles, tempdir, comptype, pldmflag
@@ -438,11 +591,14 @@ class FwpkgCommand:
         """
 
         for component in components:
+            component_l = component.lower()
             if (
-                component.endswith(".fwpkg")
-                or component.lower().endswith(".hpb")
-                or component.lower().endswith(".fup")
-                or component.endswith(".zip")
+                component_l.endswith(".fwpkg")
+                or component_l.endswith(".hpb")
+                or component_l.endswith(".bin")
+                or component_l.endswith(".fup")
+                or component_l.endswith(".pup")
+                or component_l.endswith(".zip")
             ):
                 uploadcommand = "--component %s" % component
             else:
@@ -454,15 +610,31 @@ class FwpkgCommand:
                 LOGGER.info("Setting --update_target --update_repository options as it is A or B")
                 uploadcommand += " --update_target --update_repository"
             elif comptype in ["BC"]:
-                LOGGER.info("Setting --update_target option as it is BC.")
-                uploadcommand += " --update_target"
+                LOGGER.info("Setting --update_target --update_repository options and " "calling uploadcomp once.")
+                uploadcommand1 = uploadcommand + " --update_repository --update_target"
+                if options.tover:
+                    LOGGER.info("Setting --tpmover if tpm enabled.")
+                    uploadcommand1 += " --tpmover"
+                ret = self.auxcommands["uploadcomp"].run(uploadcommand1)
+                if ret != ReturnCodes.SUCCESS:
+                    raise UploadError("Error uploading component.")
+                LOGGER.info(
+                    "Continuing the flow with --component such that uploadcomp"
+                    " is called again to upload component to the repository."
+                )
             if options.update_srs:
                 LOGGER.info("Setting --update_srs to store as recovery set.")
                 uploadcommand += " --update_srs"
+            if options.response:
+                LOGGER.info("Setting --response for extended UpdateService Status")
+                uploadcommand += " --response"
             if options.tover:
                 LOGGER.info("Setting --tpmover if tpm enabled.")
                 uploadcommand += " --tpmover"
-            self.rdmc.ui.printer("Uploading firmware: %s\n" % os.path.basename(component))
+            if "update_repository" not in uploadcommand:
+                self.rdmc.ui.printer("Uploading firmware: %s\n" % os.path.basename(component))
+            if "update_target" in uploadcommand:
+                self.rdmc.ui.printer("Flashing firmware: %s\n" % os.path.basename(component))
             try:
                 ret = self.auxcommands["uploadcomp"].run(uploadcommand)
                 if ret != ReturnCodes.SUCCESS:
@@ -536,7 +708,7 @@ class FwpkgCommand:
 
         self.cmdbase.add_login_arguments_group(customparser)
 
-        customparser.add_argument("fwpkg", help="""fwpkg file path""", metavar="[FWPKG]")
+        customparser.add_argument("fwpkg", help="""fwpkg file path""", metavar="[FWPKG]", default=None)
         customparser.add_argument(
             "--forceupload",
             dest="forceupload",
@@ -551,6 +723,20 @@ class FwpkgCommand:
             help="Add this flag to ignore all checks to the taskqueue before attempting to process the .fwpkg file.",
             default=False,
         )
+        customparser.add_argument(
+            "--enable_thirdparty_fw",
+            dest="enable_thirdparty_fw",
+            action="store_true",
+            help="Add this flag to enable flashing third party firmware to server.",
+            default=False,
+        )
+        customparser.add_argument(
+            "--disable_thirdparty_fw",
+            dest="disable_thirdparty_fw",
+            action="store_true",
+            help="Add this flag to disable flashing third party firmware to server.",
+            default=False,
+        )
         customparser.add_argument(
             "--tpmover",
             dest="tover",
@@ -566,6 +752,13 @@ class FwpkgCommand:
             "NOTE: This requires an account login with the system recovery set privilege.",
             default=False,
         )
+        customparser.add_argument(
+            "--response",
+            dest="response",
+            action="store_true",
+            help="Use this flag to return the iLO response body.",
+            default=False,
+        )
         customparser.add_argument(
             "--targets",
             help="If targets value specify a comma separated\t" "firmwareinventory id only",
