@@ -15,7 +15,7 @@
 # ##
 
 # -*- coding: utf-8 -*-
-""" Upload Component Command for rdmc """
+"""Upload Component Command for rdmc"""
 
 import os
 import json
@@ -46,6 +46,7 @@ from redfish.hpilo.risblobstore2 import BlobStore2
 
 try:
     from rdmc_helper import (
+        UI,
         LOGGER,
         ReturnCodes,
         InvalidCommandLineErrorOPTS,
@@ -56,6 +57,7 @@ try:
     )
 except ImportError:
     from ilorest.rdmc_helper import (
+        UI,
         LOGGER,
         ReturnCodes,
         InvalidCommandLineErrorOPTS,
@@ -103,6 +105,7 @@ class UploadComponentCommand:
         self.cmdbase = None
         self.rdmc = None
         self.auxcommands = dict()
+        self.status_content = {}
 
     def run(self, line, help_disp=False):
         """Wrapper function for upload command main function
@@ -129,9 +132,14 @@ class UploadComponentCommand:
             options.component.endswith(".fwpkg")
             or options.component.endswith(".fup")
             or options.component.endswith(".hpb")
+            or options.component.endswith(".HPb")
+            or options.component.endswith(".pup")
         ):
             fwpkg = True
-            comp, loc, ctype, pldmfw = self.auxcommands["flashfwpkg"].preparefwpkg(self, options.component)
+            command_name = self.cmdbase.name
+            comp, loc, ctype, pldmfw = self.auxcommands["flashfwpkg"].preparefwpkg(
+                self, options.component, command_name
+            )
             # if pldm firmware
             if pldmfw:
                 path = self.rdmc.app.typepath.defs.systempath
@@ -164,6 +172,7 @@ class UploadComponentCommand:
                 ret = self.uploadfunction(filestoupload, options)
 
             if ret == ReturnCodes.SUCCESS and not options.update_target:
+
                 self.rdmc.ui.printer("Uploading took %s\n" % human_readable_time(time.time() - start_time))
 
             if len(filestoupload) > 1:
@@ -179,6 +188,7 @@ class UploadComponentCommand:
             ret = ReturnCodes.SUCCESS
         else:
             ret = ReturnCodes.FAILED_TO_UPLOAD_COMPONENT
+
         self.cmdbase.logout_routine(self, options)
         # Return code
         return ret
@@ -423,7 +433,10 @@ class UploadComponentCommand:
                 output = fle.read()
             data.append(("file", (ilo_upload_filename, output, "application/octet-stream")))
 
-            self.rdmc.ui.printer("Uploading component " + filename + ".\n")
+            if options.update_repository:
+                self.rdmc.ui.printer("Uploading component " + filename + ".\n")
+            if options.update_target:
+                self.rdmc.ui.printer("Flashing component " + filename + ".\n")
             res = self.rdmc.app.post_handler(
                 str(urltosend),
                 data,
@@ -442,14 +455,33 @@ class UploadComponentCommand:
             if res.status != 200:
                 return ReturnCodes.FAILED_TO_UPLOAD_COMPONENT
             else:
-                self.rdmc.ui.printer("Component " + filename + " uploading successfully.\n")
+                if options.update_repository:
+                    self.rdmc.ui.printer("Component " + filename + " uploading successfully.\n")
+                if options.update_target:
+                    self.rdmc.ui.printer("Component " + filename + " flashing successfully.\n")
 
             if not self.wait_for_state_change():
+                if options.response:
+                    self.rdmc.ui.printer("UpdateService Status:")
+                    UI().print_out_json(self.status_content)
+                LOGGER.info("\nUpdate Service Status: {}".format(self.status_content))
                 # Failed to upload the component.
                 raise UploadError("Error while processing the component.")
 
+        if options.response:
+            self.rdmc.ui.printer("UpdateService Status:")
+            UI().print_out_json(self.status_content)
+        LOGGER.info("\nUpdate Service Status: {}".format(self.status_content))
+
         return ReturnCodes.SUCCESS
 
+    def get_update_service_status(self, result):
+        if "Result" in result["Oem"]["Hpe"]:
+            res = "MessageId:" + result["Oem"]["Hpe"]["Result"]["MessageId"]
+            complete_res = "{" + res + "}"
+            self.status_content["Result"] = complete_res
+        self.status_content["State"] = (result["Oem"]["Hpe"]["State"]).upper()
+
     def wait_for_state_change(self, wait_time=4800):
         """Wait for the iLO UpdateService to a move to terminal state.
         :param options: command line options
@@ -458,14 +490,16 @@ class UploadComponentCommand:
         :type wait_time: int.
         """
         total_time = 0
+        result = dict()
         spinner = ["|", "/", "-", "\\"]
         state = ""
         self.rdmc.ui.printer("Waiting for iLO UpdateService to finish processing the component\n")
 
         while total_time < wait_time:
-            state, _ = self.get_update_service_state()
+            state, result = self.get_update_service_state()
 
             if state == "ERROR":
+                self.get_update_service_status(result)
                 return False
             elif state != "COMPLETED" and state != "IDLE" and state != "COMPLETE":
                 # Lets try again after 8 seconds
@@ -479,6 +513,7 @@ class UploadComponentCommand:
 
                 total_time += 8
             else:
+                self.get_update_service_status(result)
                 break
 
         if total_time > wait_time:
@@ -632,7 +667,10 @@ class UploadComponentCommand:
                     else:
                         dispatchflag = ctypes.c_uint32(0x00000001 | 0x00000004 | 0x00000010 | 0x00000020)
 
-                self.rdmc.ui.printer("Uploading component " + filename + "\n")
+                if options.update_repository:
+                    self.rdmc.ui.printer("Uploading component " + filename + "\n")
+                if options.update_target:
+                    self.rdmc.ui.printer("Flashing component " + filename + "\n")
 
                 ret = dll.uploadComponent(
                     ctypes.create_string_buffer(compsigpath.encode("utf-8")),
@@ -648,10 +686,17 @@ class UploadComponentCommand:
                     upload_failed = True
                 else:
                     LOGGER.info("Component {} uploaded successfully".format(filename))
-                    self.rdmc.ui.printer("Component " + filename + " uploaded successfully.\n")
+                    if options.update_repository:
+                        self.rdmc.ui.printer("Component " + filename + " uploaded successfully.\n")
+                    if options.update_target:
+                        self.rdmc.ui.printer("Component " + filename + " flashed successfully.\n")
                     self.rdmc.ui.printer("[200] The operation completed successfully.\n")
                     if not options.update_target:
                         if not self.wait_for_state_change():
+                            if options.response:
+                                self.rdmc.ui.printer("UpdateService Status:")
+                                UI().print_out_json(self.status_content)
+                            LOGGER.info("\nUpdate Service Status: {}".format(self.status_content))
                             # Failed to upload the component.
                             raise UploadError("Error while processing the component.")
 
@@ -671,6 +716,15 @@ class UploadComponentCommand:
             LOGGER.error("Exception occured, {}".format(excep))
             raise excep
 
+        if options.response:
+            path = "/redfish/v1/UpdateService"
+            results = self.rdmc.app.get_handler(path, service=True, silent=True)
+            res = results.dict
+            self.get_update_service_status(res)
+            self.rdmc.ui.printer("UpdateService Status:")
+            UI().print_out_json(self.status_content)
+        LOGGER.info("\nUpdate Service Status: {}".format(self.status_content))
+
         if upload_failed:
             return ReturnCodes.FAILED_TO_UPLOAD_COMPONENT
         else:
@@ -741,6 +795,13 @@ class UploadComponentCommand:
             " structure makes the information easier to parse.",
             default=False,
         )
+        customparser.add_argument(
+            "--response",
+            dest="response",
+            action="store_true",
+            help="Use this flag to return the iLO response body.",
+            default=False,
+        )
         customparser.add_argument(
             "--component",
             dest="component",
