@@ -1,5 +1,5 @@
 ###
-# Copyright 2017 Hewlett Packard Enterprise, Inc. All rights reserved.
+# Copyright 2017-2025 Hewlett Packard Enterprise, Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,8 +18,6 @@
 """This is the helper module for RDMC"""
 
 # ---------Imports---------
-from __future__ import unicode_literals
-
 import os
 import sys
 from argparse import (
@@ -29,31 +27,16 @@ from argparse import (
     RawDescriptionHelpFormatter,
     _ArgumentGroup,
 )
-from builtins import bytes, int, object, str, super
-
 from redfish.ris import NothingSelectedError
 
 try:
-    import cliutils
-except ImportError:
-    from ilorest import cliutils
-
-try:
-    import versioning
-except ImportError:
-    from ilorest import versioning
-try:
-    import rdmc_helper
-except ImportError:
-    from ilorest import rdmc_helper
-
-try:
-    from rdmc_helper import InvalidCommandLineErrorOPTS
-except ImportError:
+    from ilorest import cliutils, versioning, rdmc_helper
     from ilorest.rdmc_helper import InvalidCommandLineErrorOPTS
-
-
-# from extensions.COMMANDS import LoginCommand, SelectCommand
+except ImportError:
+    import cliutils
+    import versioning
+    import rdmc_helper
+    from rdmc_helper import InvalidCommandLineErrorOPTS
 
 # ---------End of imports---------
 
@@ -94,8 +77,6 @@ class CommandBase(object):
             self.parser = argparser
 
         self.parser.usage = usage
-        # TODO: See if we can remove this or stop it from opening a subprocess all the time
-        # self._cli = cliutils.CLI()
 
     def run(self, line, help_disp=False):
         """Called to actually perform the work.
@@ -195,7 +176,7 @@ class RdmcCommandBase(CommandBase):
             if getattr(options, "sessionid", False):
                 inputline.extend(["--sessionid", options.sessionid])
 
-            logobj.loginfunction(inputline)
+            logobj.loginfunction(inputline, skipbuild=skipbuild)
             loggedin = True
 
         if not (loggedin or client or options.url or cmdinstance.rdmc.app.typepath.url):
@@ -222,6 +203,13 @@ class RdmcCommandBase(CommandBase):
         """
         inputline = list()
 
+        if cmdinstance.ident["name"] == "detectilo" or cmdinstance.ident["name"] == "appaccount":
+            try:
+                client = cmdinstance.rdmc.app.current_client
+                return client
+            except Exception:
+                return None
+
         self.login_validation(cmdinstance, options, skipbuild=skipbuild)
         logobj = cmdinstance.rdmc.load_command(cmdinstance.rdmc.search_commands("LoginCommand"))
         selobj = cmdinstance.rdmc.load_command(cmdinstance.rdmc.search_commands("SelectCommand"))
@@ -454,7 +442,9 @@ class RdmcOptionParser(ArgumentParser):
             "--debug",
             dest="debug",
             action="store_true",
-            help="""Display debug information.""",
+            help="""[DEPRECATED] This flag is deprecated and will be removed in future versions.
+            Use '--logconfig' for enabling debug logs.
+            Enable debug mode for detailed logging output.""",
             default=False,
         )
         self.add_argument(
@@ -468,7 +458,18 @@ class RdmcOptionParser(ArgumentParser):
             "--nostdoutlog",
             dest="nostdoutlog",
             action="store_true",
-            help="""Disable debug logs to stdout.""",
+            help="""[DEPRECATED] This flag is deprecated and will be removed in future versions.
+            Use '--logconfig' for enabling debug logs.
+            Enable debug mode for detailed logging output.""",
+            default=False,
+        )
+        self.add_argument(
+            "--noinfolog",
+            "--nodefaultlog",
+            "--no_default_log",
+            dest="noinfolog",
+            action="store_true",
+            help="""Disable default INFO logs.""",
             default=False,
         )
         self.add_argument(
